from typing import Dict


class CustomImages(object):
    def __init__(
        self,
        custom_images,    # type: Dict
    ):
        # type: (...) -> CustomImages

        self.custom_images = custom_images
