
class PaymentType(object):

    def __init__(
            self,
            name=None,                  # type: str
            payment_name=None,          # type: str
            payment_type=None,          # type: int
            image_path=None,            # type: str
            electronic_type_id=None     # type: int
    ):
        # type: (...) -> None

        self.name = name
        self.payment_name = payment_name
        self.payment_type = payment_type
        self.image_path = image_path
        self.electronic_type_id = electronic_type_id
