import os
import json


class GetUpSellPartCodesResponse(object):
    def __init__(
            self,
            up_sell_part_codes_path  # type: str
    ):
        # type: (...) -> None

        self.up_sell_part_codes = json.loads("{}")
        if not up_sell_part_codes_path or not os.path.exists(up_sell_part_codes_path):
            return

        with open(up_sell_part_codes_path) as f:
            config_file = f.read()
            self.up_sell_part_codes = json.loads(config_file or "{}")
