from messagebus import TokenCreator, TokenPriority


class NDiscountTokens(object):
    NDISCOUNT_MESSAGE_GROUP = "112"

    TK_NDISCOUNT_ADD_BENEFIT = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "1")
    TK_NDISCOUNT_REMOVE = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "2")
    TK_NDISCOUNT_APPLY_AUTOMATIC_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "6")
    TK_NDISCOUNT_GET_BENEFITS = TokenCreator.create_token(TokenPriority.low, NDISCOUNT_MESSAGE_GROUP, "7")
