from _CancelPaymentProcessor import CancelPaymentProcessor
from _CancelPaymentOrderProcessor import CancelPaymentOrderProcessor
from _ConfirmFinishPaymentProcessor import ConfirmFinishPaymentProcessor
from _FinishPaymentProcessor import FinishPaymentProcessor
from _GetDocumentProcessor import GetDocumentProcessor
from _GetElectronicTypesProcessor import GetElectronicTypesProcessor
from _StartPaymentProcessor import StartPaymentProcessor
from _TestConnectionProcessor import TestConnectionProcessor
from _GetNameProcessor import GetNameProcessor
from _CacheProcessor import CacheProcessor
