from messagebus import TokenCreator, TokenPriority


class ResponseToken(object):
    NCOUPON_MESSAGE_GROUP = "113"

    TK_NCOUPON_ERROR = TokenCreator.create_token(TokenPriority.low, NCOUPON_MESSAGE_GROUP, "F001")
