"""
Contains information about the dateutil version.
"""

VERSION_MAJOR = 2
VERSION_MINOR = 6
VERSION_PATCH = 1

VERSION_TUPLE = (VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH)
VERSION = '.'.join(map(str, VERSION_TUPLE))
