# encoding: utf-8

from typing import Any


class JustificationDiscount(object):
    def __init__(
        self,
        row_database,               # type: Any
        benefit_id,                 # type: str
        justification,              # type: str
        applied_by,                 # type: str
        benefit_total_value,        # type: float
    ):
        # type: (...) -> None

        self.order_id = row_database.get_entry("OrderId")
        self.order_date = row_database.get_entry("OrderDate")
        self.order_time = row_database.get_entry("OrderTime")
        self.order_value = row_database.get_entry("TotalGross")
        self.benefit_id = benefit_id
        self.justification = justification
        self.applied_by = applied_by
        self.benefit_total_value = benefit_total_value
