from typing import List


class StoreGoals(object):
    def __init__(
        self,
        amount_goal,            # type: int
        average_sale_goal,      # type: int
        operator_sale_goal,     # type: int
        item_goals,             # type: List
    ):
        # type: (...) -> None

        self.amount_goal = amount_goal
        self.average_sale_goal = average_sale_goal
        self.operator_sale_goal = operator_sale_goal
        self.sold_quantity = 0
        self.item_goals = item_goals
