class AppliedBenefitDto(object):
    def __init__(
        self,               # type: AppliedBenefitDto
        response_detail,    # type: str
        benefit_applied,    # type: bool
    ):
        # type: (...) -> None

        self.response_detail = response_detail
        self.benefit_applied = benefit_applied
