from typing import List

from ._DeliveryHeaderDto import DeliveryHeaderDto
from ._DeliveryBodyDto import DeliveryBodyDto


class DeliveryGeneratorDto(object):
    def __init__(
        self,
        header,                 # type: DeliveryHeaderDto
        body,                   # type: DeliveryBodyDto
        products,               # type: object
        cfh_products,           # type: List
        show_customer_phone,    # type: bool
    ):
        # type: (...) -> None

        self.header = header
        self.body = body
        self.products = products
        self.cfh_products = cfh_products
        self.show_customer_phone = show_customer_phone
