from application.interactor.apply.dto import ApplyContext


class ApplyBenefitRequest(object):
    def __init__(
        self,
        benefit,  # type: str,
        context,  # type: ApplyContext
    ):
        # type: (...) -> None

        self.benefit = benefit
        self.context = context
