from ._TableReportOrderDto import TableReportOrderDto  # noqa
from typing import List, Dict


class TableReportBodyDto(object):
    def __init__(self):
        self.total_amount = 0.0  # type: float
        self.sub_total_amount = 0.0  # type: float
        self.discount_amount = 0.0  # type: float
        self.detailed_discounts = {}  # type: Dict
        self.detailed_payments = {}  # type: Dict
        self.total_after_discount = 0.0  # type: float
        self.total_tender = 0.0  # type: float
        self.total_per_seat = 0.0  # type: float
        self.orders = []  # type: List[TableReportOrderDto]
        self.service_seats = 0  # type: int
        self.order_id = None  # type: str
        self.tip = None  # type: float
        self.promotional_tenders = 0  # type: float
        self.detailed_tip_excluded = {}  # type: Dict
        self.service_tenders = []  # type: List[dict]
        self.discount_groups = {}  # type: Dict
