from typing import List


class FiscalOrderResponse(object):
    def __init__(
        self,
        fiscal_coupon,          # type: str
        tef_customer_receipt,   # type: List[str]
    ):
        # type: (...) -> None

        self.fiscalCoupon = fiscal_coupon
        self.tefCustomerReceipt = tef_customer_receipt

