from typing import Dict


class AdditionalInfo(object):
    def __init__(
        self,
        additional_info,    # type: Dict
    ):
        # type: (...) -> None

        self.welcome_message = additional_info.get("welcomeMessage", None)
        self.end_sale_message = additional_info.get("endSaleMessage", None)
